@@ -20,6 +20,12 @@ attribute. Thus the model can be viewed as a special part of the lookup
 mechanism.
 """
 
+try:
+    from functools import lru_cache
+except ImportError:
+    from backports.functools_lru_cache import lru_cache
+
+import itertools
 import pprint
 import os
 import types
@@ -88,6 +94,7 @@ class ObjectModel(object):
     def __contains__(self, name):
         return name in self.attributes()
 
+    @lru_cache(maxsize=None)
     def attributes(self):
         """Get the attributes which are exported by this object model."""
         return [obj[2:] for obj in dir(self) if obj.startswith('py')]
@@ -226,9 +233,16 @@ class FunctionModel(ObjectModel):
             returns = self._instance.returns
 
         args = self._instance.args
-        annotations = {arg.name: annotation
-                       for (arg, annotation) in zip(args.args, args.annotations)
-                       if annotation}
+        pair_annotations = itertools.chain(
+            six.moves.zip(args.args, args.annotations),
+            six.moves.zip(args.kwonlyargs, args.kwonlyargs_annotations)
+        )
+
+        annotations = {
+            arg.name: annotation
+            for (arg, annotation) in pair_annotations
+            if annotation
+        }
         if args.varargannotation:
             annotations[args.vararg] = args.varargannotation
         if args.kwargannotation:
