@@ -1,4 +1,4 @@
-from __future__ import print_statement
+from __future__ import print_function
 from data import SSL1
 class MyConnection(SSL1.Connection):
 
