@@ -1,20 +1,13 @@
-# copyright 2003-2014 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
-# contact http://www.logilab.fr/ -- mailto:contact@logilab.fr
-#
-# This file is part of astroid.
-#
-# astroid is free software: you can redistribute it and/or modify it
-# under the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 2.1 of the License, or (at your
-# option) any later version.
-#
-# astroid is distributed in the hope that it will be useful, but
-# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
-# for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License along
-# with astroid. If not, see <http://www.gnu.org/licenses/>.
+# Copyright (c) 2006-2014 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
+# Copyright (c) 2011, 2013-2015 Google, Inc.
+# Copyright (c) 2013-2016 Claudiu Popa <pcmanticore@gmail.com>
+# Copyright (c) 2015-2016 Cara Vinson <ceridwenv@gmail.com>
+# Copyright (c) 2015 Philip Lorenz <philip@bithub.de>
+# Copyright (c) 2015 Rene Zhang <rz99@cornell.edu>
+
+# Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
+# For details: https://github.com/PyCQA/astroid/blob/master/COPYING.LESSER
+
 """tests for specific behaviour of astroid scoped nodes (i.e. module, class and
 function)
 """
@@ -29,9 +22,10 @@ from astroid import nodes
 from astroid import scoped_nodes
 from astroid import util
 from astroid.exceptions import (
-    InferenceError, NotFoundError,
+    InferenceError, AttributeInferenceError,
     NoDefault, ResolveError, MroError,
     InconsistentMroError, DuplicateBasesError,
+    TooManyLevelsError,
 )
 from astroid.bases import (
     BUILTINS, Instance,
@@ -75,7 +69,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
                          os.path.abspath(resources.find('data/module.py')))
         self.assertEqual(len(self.module.getattr('__dict__')), 1)
         self.assertIsInstance(self.module.getattr('__dict__')[0], nodes.Dict)
-        self.assertRaises(NotFoundError, self.module.getattr, '__path__')
+        self.assertRaises(AttributeInferenceError, self.module.getattr, '__path__')
         self.assertEqual(len(self.pack.getattr('__path__')), 1)
         self.assertIsInstance(self.pack.getattr('__path__')[0], nodes.List)
 
@@ -101,7 +95,6 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(cnx.name, 'Connection')
         self.assertEqual(cnx.root().name, 'data.SSL1.Connection1')
         self.assertEqual(len(self.nonregr.getattr('enumerate')), 2)
-        # raise ResolveError
         self.assertRaises(InferenceError, self.nonregr.igetattr, 'YOAA')
 
     def test_wildcard_import_names(self):
@@ -121,7 +114,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         __all__ = 'Aaa', '_bla', 'name'
         ''')
         values = sorted(['Aaa', 'name', 'other', 'func'])
-        self.assertEqual(sorted(m._public_names()), values)
+        self.assertEqual(sorted(m.public_names()), values)
         m = builder.parse('''
         name = 'a'
         _bla = 2
@@ -130,7 +123,7 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
 
         def func(): return 'yo'
         ''')
-        res = sorted(m._public_names())
+        res = sorted(m.public_names())
         self.assertEqual(res, values)
 
         m = builder.parse('''
@@ -138,14 +131,14 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
             trop = "test"
             __all__ = (trop, "test1", tzop, 42)
         ''')
-        res = sorted(m._public_names())
+        res = sorted(m.public_names())
         self.assertEqual(res, ["trop", "tzop"])
 
         m = builder.parse('''
             test = tzop = 42
             __all__ = ('test', ) + ('tzop', )
         ''')
-        res = sorted(m._public_names())
+        res = sorted(m.public_names())
         self.assertEqual(res, ['test', 'tzop'])
 
     def test_module_getattr(self):
@@ -183,6 +176,18 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
         modname = mod.relative_to_absolute_name('', 1)
         self.assertEqual(modname, 'very.multi')
 
+    def test_relative_to_absolute_name_beyond_top_level(self):
+        mod = nodes.Module('a.b.c', '')
+        mod.package = True
+        for level in (5, 4):
+            with self.assertRaises(TooManyLevelsError) as cm:
+                mod.relative_to_absolute_name('test', level)
+
+            expected = ("Relative import with too many levels "
+                        "({level}) for module {name!r}".format(
+                            level=level - 1, name=mod.name))
+            self.assertEqual(expected, str(cm.exception))
+
     def test_import_1(self):
         data = '''from . import subpackage'''
         sys.path.insert(0, resources.find('data'))
@@ -239,8 +244,8 @@ class ModuleNodeTest(ModuleLoader, unittest.TestCase):
             # only Module.stream as the recommended way to retrieve
             # its file stream.
             with warnings.catch_warnings(record=True) as cm:
-                warnings.simplefilter("always")
-                self.assertIsNot(astroid.file_stream, astroid.file_stream)
+                with test_utils.enable_warning(PendingDeprecationWarning):
+                    self.assertIsNot(astroid.file_stream, astroid.file_stream)
             self.assertGreater(len(cm), 1)
             self.assertEqual(cm[0].category, PendingDeprecationWarning)
 
@@ -263,7 +268,8 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(func.getattr('__name__')[0].value, 'make_class')
         self.assertEqual(len(func.getattr('__doc__')), 1)
         self.assertIsInstance(func.getattr('__doc__')[0], nodes.Const)
-        self.assertEqual(func.getattr('__doc__')[0].value, 'check base is correctly resolved to Concrete0')
+        self.assertEqual(func.getattr('__doc__')[0].value,
+                         'check base is correctly resolved to Concrete0')
         self.assertEqual(len(self.module.getattr('__dict__')), 1)
         self.assertIsInstance(self.module.getattr('__dict__')[0], nodes.Dict)
 
@@ -307,12 +313,11 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         '''
         tree = builder.parse(code)
         func = tree['nested_args']
-        self.assertEqual(sorted(func._locals), ['a', 'b', 'c', 'd'])
+        self.assertEqual(sorted(func.locals), ['a', 'b', 'c', 'd'])
         self.assertEqual(func.args.format_args(), 'a, (b, c, d)')
 
     def test_four_args(self):
         func = self.module['four_args']
-        #self.assertEqual(func.args.args, ['a', ('b', 'c', 'd')])
         local = sorted(func.keys())
         self.assertEqual(local, ['a', 'b', 'c', 'd'])
         self.assertEqual(func.type, 'function')
@@ -324,6 +329,15 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         func = self.module['four_args']
         self.assertEqual(func.args.format_args(), 'a, b, c, d')
 
+    @test_utils.require_version('3.0')
+    def test_format_args_keyword_only_args(self):
+        node = builder.parse('''
+        def test(a: int, *, b: dict):
+            pass
+        ''').body[-1].args
+        formatted = node.format_args()
+        self.assertEqual(formatted, 'a:int, *, b:dict')
+
     def test_is_generator(self):
         self.assertTrue(self.module2['generator'].is_generator())
         self.assertFalse(self.module2['not_a_generator'].is_generator())
@@ -341,7 +355,7 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
         self.assertFalse(func.is_abstract(pass_is_abstract=False))
 
     def test_is_abstract_decorated(self):
-        methods = test_utils.extract_node("""
+        methods = builder.extract_node("""
             import abc
 
             class Klass(object):
@@ -473,14 +487,14 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                 def stcmethod(cls):
                     pass
         """)
-        node = astroid._locals['Node'][0]
-        self.assertEqual(node._locals['clsmethod_subclass'][0].type,
+        node = astroid.locals['Node'][0]
+        self.assertEqual(node.locals['clsmethod_subclass'][0].type,
                          'classmethod')
-        self.assertEqual(node._locals['clsmethod'][0].type,
+        self.assertEqual(node.locals['clsmethod'][0].type,
                          'classmethod')
-        self.assertEqual(node._locals['staticmethod_subclass'][0].type,
+        self.assertEqual(node.locals['staticmethod_subclass'][0].type,
                          'staticmethod')
-        self.assertEqual(node._locals['stcmethod'][0].type,
+        self.assertEqual(node.locals['stcmethod'][0].type,
                          'staticmethod')
 
     def test_decorator_builtin_descriptors(self):
@@ -546,28 +560,28 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
                 def long_classmethod(cls): 
                     pass
         """)
-        node = astroid._locals['SomeClass'][0]
-        self.assertEqual(node._locals['static'][0].type,
+        node = astroid.locals['SomeClass'][0]
+        self.assertEqual(node.locals['static'][0].type,
                          'staticmethod')
-        self.assertEqual(node._locals['classmethod'][0].type,
+        self.assertEqual(node.locals['classmethod'][0].type,
                          'classmethod')
-        self.assertEqual(node._locals['not_so_static'][0].type,
+        self.assertEqual(node.locals['not_so_static'][0].type,
                          'method')
-        self.assertEqual(node._locals['not_so_classmethod'][0].type,
+        self.assertEqual(node.locals['not_so_classmethod'][0].type,
                          'method')
-        self.assertEqual(node._locals['classmethod_wrapped'][0].type,
+        self.assertEqual(node.locals['classmethod_wrapped'][0].type,
                          'classmethod')
-        self.assertEqual(node._locals['staticmethod_wrapped'][0].type,
+        self.assertEqual(node.locals['staticmethod_wrapped'][0].type,
                          'staticmethod')
-        self.assertEqual(node._locals['long_classmethod'][0].type,
+        self.assertEqual(node.locals['long_classmethod'][0].type,
                          'classmethod')
 
     def test_igetattr(self):
-        func = test_utils.extract_node('''
+        func = builder.extract_node('''
         def test():
             pass
         ''')
-        func._instance_attrs['value'] = [nodes.Const(42)]
+        func.instance_attrs['value'] = [nodes.Const(42)]
         value = func.getattr('value')
         self.assertEqual(len(value), 1)
         self.assertIsInstance(value[0], nodes.Const)
@@ -578,16 +592,27 @@ class FunctionNodeTest(ModuleLoader, unittest.TestCase):
 
     @test_utils.require_version(minver='3.0')
     def test_return_annotation_is_not_the_last(self):
-        func = builder.parse('''
+        func = builder.extract_node('''
         def test() -> bytes:
             pass
             pass
             return
-        ''').body[0]
+        ''')
         last_child = func.last_child()
         self.assertIsInstance(last_child, nodes.Return)
         self.assertEqual(func.tolineno, 5)
 
+    @test_utils.require_version(minver='3.6')
+    def test_method_init_subclass(self):
+        klass = builder.extract_node('''
+        class MyClass:
+            def __init_subclass__(cls):
+                pass
+        ''')
+        method = klass['__init_subclass__']
+        self.assertEqual([n.name for n in method.args.args], ['cls'])
+        self.assertEqual(method.type, 'classmethod')
+
 
 class ClassNodeTest(ModuleLoader, unittest.TestCase):
 
@@ -608,7 +633,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(cls.getattr('__module__')[0].value, 'data.module')
         self.assertEqual(len(cls.getattr('__dict__')), 1)
         if not cls.newstyle:
-            self.assertRaises(NotFoundError, cls.getattr, '__mro__')
+            self.assertRaises(AttributeInferenceError, cls.getattr, '__mro__')
         for cls in (nodes.List._proxied, nodes.Const(1)._proxied):
             self.assertEqual(len(cls.getattr('__bases__')), 1)
             self.assertEqual(len(cls.getattr('__name__')), 1)
@@ -619,7 +644,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             self.assertEqual(len(cls.getattr('__mro__')), 1)
 
     def test__mro__attribute(self):
-        node = test_utils.extract_node('''
+        node = builder.extract_node('''
         class A(object): pass
         class B(object): pass
         class C(A, B): pass        
@@ -629,7 +654,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(mro.elts, node.mro())
 
     def test__bases__attribute(self):
-        node = test_utils.extract_node('''
+        node = builder.extract_node('''
         class A(object): pass
         class B(object): pass
         class C(A, B): pass
@@ -643,20 +668,20 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
 
     def test_cls_special_attributes_2(self):
         astroid = builder.parse('''
-            class A: pass
-            class B: pass
+            class A(object): pass
+            class B(object): pass
 
             A.__bases__ += (B,)
         ''', __name__)
         self.assertEqual(len(astroid['A'].getattr('__bases__')), 2)
-        self.assertIsInstance(astroid['A'].getattr('__bases__')[0], nodes.Tuple)
-        self.assertIsInstance(astroid['A'].getattr('__bases__')[1], nodes.AssignAttr)
+        self.assertIsInstance(astroid['A'].getattr('__bases__')[1], nodes.Tuple)
+        self.assertIsInstance(astroid['A'].getattr('__bases__')[0], nodes.AssignAttr)
 
     def test_instance_special_attributes(self):
         for inst in (Instance(self.module['YO']), nodes.List(), nodes.Const(1)):
-            self.assertRaises(NotFoundError, inst.getattr, '__mro__')
-            self.assertRaises(NotFoundError, inst.getattr, '__bases__')
-            self.assertRaises(NotFoundError, inst.getattr, '__name__')
+            self.assertRaises(AttributeInferenceError, inst.getattr, '__mro__')
+            self.assertRaises(AttributeInferenceError, inst.getattr, '__bases__')
+            self.assertRaises(AttributeInferenceError, inst.getattr, '__name__')
             self.assertEqual(len(inst.getattr('__dict__')), 1)
             self.assertEqual(len(inst.getattr('__doc__')), 1)
 
@@ -752,7 +777,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         method_locals = klass2.local_attr('method')
         self.assertEqual(len(method_locals), 1)
         self.assertEqual(method_locals[0].name, 'method')
-        self.assertRaises(NotFoundError, klass2.local_attr, 'nonexistant')
+        self.assertRaises(AttributeInferenceError, klass2.local_attr, 'nonexistent')
         methods = {m.name for m in klass2.methods()}
         self.assertTrue(methods.issuperset(expected_methods))
 
@@ -810,7 +835,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         '''
         astroid = builder.parse(data, __name__)
         cls = astroid['WebAppObject']
-        self.assertEqual(sorted(cls._locals.keys()),
+        self.assertEqual(sorted(cls.locals.keys()),
                          ['appli', 'config', 'registered', 'schema'])
 
     def test_class_getattr(self):
@@ -1009,9 +1034,9 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                class D: pass
         """)
         a = astroid['A']
-        b = a._locals['B'][0]
+        b = a.locals['B'][0]
         c = astroid['C']
-        d = c._locals['D'][0]
+        d = c.locals['D'][0]
         self.assertEqual(a.metaclass().name, 'ABCMeta')
         self.assertFalse(b.newstyle)
         self.assertIsNone(b.metaclass())
@@ -1063,7 +1088,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
                 self.assertEqual(meta.name, metaclass)
 
     def test_metaclass_type(self):
-        klass = test_utils.extract_node("""
+        klass = builder.extract_node("""
             def with_metaclass(meta, base=object):
                 return meta("NewBase", (base, ), {})
 
@@ -1075,7 +1100,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             [base.name for base in klass.ancestors()])
 
     def test_no_infinite_metaclass_loop(self):
-        klass = test_utils.extract_node("""
+        klass = builder.extract_node("""
             class SSS(object):
 
                 class JJJ(object):
@@ -1097,7 +1122,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertIn('JJJ', ancestors)
 
     def test_no_infinite_metaclass_loop_with_redefine(self):
-        nodes = test_utils.extract_node("""
+        ast_nodes = builder.extract_node("""
             import datetime
 
             class A(datetime.date): #@
@@ -1111,11 +1136,11 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             datetime.date = A
             datetime.date = B
         """)
-        for klass in nodes:
+        for klass in ast_nodes:
             self.assertEqual(None, klass.metaclass())
 
     def test_metaclass_generator_hack(self):
-        klass = test_utils.extract_node("""
+        klass = builder.extract_node("""
             import six
 
             class WithMeta(six.with_metaclass(type, object)): #@
@@ -1128,7 +1153,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             'type', klass.metaclass().name)
 
     def test_using_six_add_metaclass(self):
-        klass = test_utils.extract_node('''
+        klass = builder.extract_node('''
         import six
         import abc
 
@@ -1142,7 +1167,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(metaclass.qname(), 'abc.ABCMeta')
 
     def test_using_invalid_six_add_metaclass_call(self):
-        klass = test_utils.extract_node('''
+        klass = builder.extract_node('''
         import six
         @six.add_metaclass()
         class Invalid(object):
@@ -1166,7 +1191,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         instance = astroid['tgts']
         # used to raise "'_Yes' object is not iterable", see
         # https://bitbucket.org/logilab/astroid/issue/17
-        self.assertEqual(list(instance.infer()), [util.YES])
+        self.assertEqual(list(instance.infer()), [util.Uninferable])
 
     def test_slots(self):
         astroid = builder.parse("""
@@ -1237,6 +1262,18 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
             module['OldStyle'].slots()
         self.assertEqual(str(cm.exception), msg)
 
+    def test_slots_for_dict_keys(self):
+        module = builder.parse('''
+        class Issue(object):
+          SlotDefaults = {'id': 0, 'id1':1}
+          __slots__ = SlotDefaults.keys()
+        ''')
+        cls = module['Issue']
+        slots = cls.slots()
+        self.assertEqual(len(slots), 2)
+        self.assertEqual(slots[0].value, 'id')
+        self.assertEqual(slots[1].value, 'id1')
+
     def test_slots_empty_list_of_slots(self):
         module = builder.parse("""
         class Klass(object):
@@ -1279,16 +1316,42 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
 
     @test_utils.require_version(maxver='3.0')
     def test_no_mro_for_old_style(self):
-        node = test_utils.extract_node("""
+        node = builder.extract_node("""
         class Old: pass""")
         with self.assertRaises(NotImplementedError) as cm:
             node.mro()
         self.assertEqual(str(cm.exception), "Could not obtain mro for "
                                             "old-style classes.")
 
+    @test_utils.require_version(maxver='3.0')
+    def test_mro_for_classes_with_old_style_in_mro(self):
+        node = builder.extract_node('''
+        class Factory:
+            pass
+        class ClientFactory(Factory):
+            pass
+        class ReconnectingClientFactory(ClientFactory):
+            pass
+        class WebSocketAdapterFactory(object):
+            pass
+        class WebSocketClientFactory(WebSocketAdapterFactory, ClientFactory):
+            pass
+        class WampWebSocketClientFactory(WebSocketClientFactory):
+            pass
+        class RetryFactory(WampWebSocketClientFactory, ReconnectingClientFactory):
+            pas
+        ''')
+        self.assertEqualMro(
+            node,
+            ['RetryFactory', 'WampWebSocketClientFactory',
+             'WebSocketClientFactory', 'WebSocketAdapterFactory', 'object',
+             'ReconnectingClientFactory', 'ClientFactory',
+             'Factory']
+        )
+
     @test_utils.require_version(maxver='3.0')
     def test_combined_newstyle_oldstyle_in_mro(self):
-        node = test_utils.extract_node('''
+        node = builder.extract_node('''
         class Old:
             pass
         class New(object):
@@ -1358,18 +1421,16 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqualMro(astroid['E1'], ['E1', 'C1', 'B1', 'A1', 'object'])
         with self.assertRaises(InconsistentMroError) as cm:
             astroid['F1'].mro()
-        self.assertEqual(str(cm.exception),
-                         "Cannot create a consistent method resolution order "
-                         "for bases (B1, C1, A1, object), "
-                         "(C1, B1, A1, object)")
-
+        A1 = astroid.getattr('A1')[0]
+        B1 = astroid.getattr('B1')[0]
+        C1 = astroid.getattr('C1')[0]
+        object_ = builder.MANAGER.astroid_cache[BUILTINS].getattr('object')[0]
+        self.assertEqual(cm.exception.mros, [[B1, C1, A1, object_],
+                                             [C1, B1, A1, object_]])
         with self.assertRaises(InconsistentMroError) as cm:
             astroid['G1'].mro()
-        self.assertEqual(str(cm.exception),
-                         "Cannot create a consistent method resolution order "
-                         "for bases (C1, B1, A1, object), "
-                         "(B1, C1, A1, object)")
-
+        self.assertEqual(cm.exception.mros, [[C1, B1, A1, object_],
+                                             [B1, C1, A1, object_]])
         self.assertEqualMro(
             astroid['PedalWheelBoat'],
             ["PedalWheelBoat", "EngineLess",
@@ -1390,12 +1451,13 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
 
         with self.assertRaises(DuplicateBasesError) as cm:
             astroid['Duplicates'].mro()
-        self.assertEqual(str(cm.exception), "Duplicates found in the mro.")
-        self.assertTrue(issubclass(cm.exception.__class__, MroError))
-        self.assertTrue(issubclass(cm.exception.__class__, ResolveError))
+        Duplicates = astroid.getattr('Duplicates')[0]
+        self.assertEqual(cm.exception.cls, Duplicates)
+        self.assertIsInstance(cm.exception, MroError)
+        self.assertIsInstance(cm.exception, ResolveError)
 
     def test_generator_from_infer_call_result_parent(self):
-        func = test_utils.extract_node("""
+        func = builder.extract_node("""
         import contextlib
 
         @contextlib.contextmanager
@@ -1407,7 +1469,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(result.parent, func)
 
     def test_type_three_arguments(self):
-        classes = test_utils.extract_node("""
+        classes = builder.extract_node("""
         type('A', (object, ), {"a": 1, "b": 2, missing: 3}) #@
         """)
         first = next(classes.infer())
@@ -1418,26 +1480,107 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(first["a"].value, 1)
         self.assertIsInstance(first["b"], nodes.Const)
         self.assertEqual(first["b"].value, 2)
-        with self.assertRaises(NotFoundError):
+        with self.assertRaises(AttributeInferenceError):
             first.getattr("missing")
 
     def test_implicit_metaclass(self):
-        cls = test_utils.extract_node("""
+        cls = builder.extract_node("""
         class A(object):
             pass
         """)
         type_cls = scoped_nodes.builtin_lookup("type")[1][0]
         self.assertEqual(cls.implicit_metaclass(), type_cls)
 
+    def test_implicit_metaclass_lookup(self):
+        cls = builder.extract_node('''
+        class A(object):
+            pass
+        ''')
+        instance = cls.instantiate_class()
+        func = cls.getattr('mro')
+        self.assertEqual(len(func), 1)
+        self.assertRaises(AttributeInferenceError, instance.getattr, 'mro')
+
+    def test_metaclass_lookup_using_same_class(self):
+        # Check that we don't have recursive attribute access for metaclass
+        cls = builder.extract_node('''
+        class A(object): pass            
+        ''')
+        self.assertEqual(len(cls.getattr('mro')), 1)
+
+    def test_metaclass_lookup_inferrence_errors(self):
+        module = builder.parse('''
+        import six
+
+        class Metaclass(type):
+            foo = lala
+
+        @six.add_metaclass(Metaclass)
+        class B(object): pass 
+        ''')
+        cls = module['B']
+        self.assertEqual(util.Uninferable, next(cls.igetattr('foo')))
+
+    def test_metaclass_lookup(self):
+        module = builder.parse('''
+        import six
+         
+        class Metaclass(type):
+            foo = 42
+            @classmethod
+            def class_method(cls):
+                pass
+            def normal_method(cls):
+                pass
+            @property
+            def meta_property(cls):
+                return 42
+            @staticmethod
+            def static():
+                pass
+
+        @six.add_metaclass(Metaclass)
+        class A(object):
+            pass
+        ''')
+        acls = module['A']
+        normal_attr = next(acls.igetattr('foo'))
+        self.assertIsInstance(normal_attr, nodes.Const)
+        self.assertEqual(normal_attr.value, 42)
+
+        class_method = next(acls.igetattr('class_method'))
+        self.assertIsInstance(class_method, BoundMethod)
+        self.assertEqual(class_method.bound, module['Metaclass'])
+
+        normal_method = next(acls.igetattr('normal_method'))
+        self.assertIsInstance(normal_method, BoundMethod)
+        self.assertEqual(normal_method.bound, module['A'])
+
+        # Attribute access for properties:
+        #   from the metaclass is a property object
+        #   from the class that uses the metaclass, the value
+        #   of the property
+        property_meta = next(module['Metaclass'].igetattr('meta_property'))
+        self.assertIsInstance(property_meta, UnboundMethod)
+        wrapping = scoped_nodes.get_wrapping_class(property_meta)
+        self.assertEqual(wrapping, module['Metaclass'])
+
+        property_class = next(acls.igetattr('meta_property'))
+        self.assertIsInstance(property_class, nodes.Const)
+        self.assertEqual(property_class.value, 42)
+
+        static = next(acls.igetattr('static'))
+        self.assertIsInstance(static, scoped_nodes.FunctionDef)
+
     @test_utils.require_version(maxver='3.0')
     def test_implicit_metaclass_is_none(self):
-        cls = test_utils.extract_node("""
+        cls = builder.extract_node("""
         class A: pass
         """)
         self.assertIsNone(cls.implicit_metaclass())
 
     def test_local_attr_invalid_mro(self):
-        cls = test_utils.extract_node("""
+        cls = builder.extract_node("""
         # A has an invalid MRO, local_attr should fallback
         # to using .ancestors.
         class A(object, object):
@@ -1486,7 +1629,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertRaises(DuplicateBasesError, module['B'].mro)
 
     def test_instance_bound_method_lambdas(self):
-        ast_nodes = test_utils.extract_node('''
+        ast_nodes = builder.extract_node('''
         class Test(object): #@
             lam = lambda self: self
             not_method = lambda xargs: xargs
@@ -1503,7 +1646,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertIsInstance(not_method, scoped_nodes.Lambda)
 
     def test_class_extra_decorators_frame_is_not_class(self):
-        ast_node = test_utils.extract_node('''
+        ast_node = builder.extract_node('''
         def ala():
             def bala(): #@
                 func = 42
@@ -1511,7 +1654,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(ast_node.extra_decorators, [])
 
     def test_class_extra_decorators_only_callfunc_are_considered(self):
-        ast_node = test_utils.extract_node('''
+        ast_node = builder.extract_node('''
         class Ala(object):
              def func(self): #@
                  pass
@@ -1520,7 +1663,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(ast_node.extra_decorators, [])
 
     def test_class_extra_decorators_only_assignment_names_are_considered(self):
-        ast_node = test_utils.extract_node('''
+        ast_node = builder.extract_node('''
         class Ala(object):
              def func(self): #@
                  pass
@@ -1531,7 +1674,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(ast_node.extra_decorators, [])
 
     def test_class_extra_decorators_only_same_name_considered(self):
-        ast_node = test_utils.extract_node('''
+        ast_node = builder.extract_node('''
         class Ala(object):
              def func(self): #@
                 pass
@@ -1541,7 +1684,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(ast_node.type, 'method')
 
     def test_class_extra_decorators(self):
-        static_method, clsmethod = test_utils.extract_node('''
+        static_method, clsmethod = builder.extract_node('''
         class Ala(object):
              def static(self): #@
                  pass
@@ -1556,7 +1699,7 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         self.assertEqual(static_method.type, 'staticmethod')
 
     def test_extra_decorators_only_class_level_assignments(self):
-        node = test_utils.extract_node('''
+        node = builder.extract_node('''
         def _bind(arg):
             return arg.bind
 
@@ -1578,6 +1721,18 @@ class ClassNodeTest(ModuleLoader, unittest.TestCase):
         parent = bind.scope()
         self.assertEqual(len(parent.extra_decorators), 0)
 
+    @test_utils.require_version(minver='3.0')
+    def test_class_keywords(self):
+        data = '''
+            class TestKlass(object, metaclass=TestMetaKlass,
+                    foo=42, bar='baz'):
+                pass
+        '''
+        astroid = builder.parse(data, __name__)
+        cls = astroid['TestKlass']
+        self.assertEqual(len(cls.keywords), 2)
+        self.assertEqual([x.arg for x in cls.keywords], ['foo', 'bar'])
+
 
 if __name__ == '__main__':
     unittest.main()
